/*
   Name:    "EuroCalcJava-oo.rex"

   Invoke:  "java com.ibm.bsf.Main -lang rexx -in EuroCalcJava-oo.rex"
                or
            "java com.ibm.bsf.Main            -in EuroCalcJava-oo.rex"
                or
             "rexxj EuroCalcJava-oo.rex"
                or
             "rexx EuroCalcJava-oo.rex"

   Purpose:  a Java solution using Java's "awt" (abstract window toolkit) matching
             the Windows OODialog-based "Euro3Calc" application, portable

   Needs:    IBM's "Bean Scripting Framework" (BSF)
             (cf. "http://oss.software.ibm.com/developerworks/projects/bsf")

             and the package "bsf4rexx"
             (cf. "http://nestroy.wi-inf.uni-essen.de/Forschung/rgf/Entwicklung.html")

             and Object Rexx

   Date:    2001-12-05 (on the way from Augsburg to Vienna)
            2003-01-23 (on the way from Augsburg to Vienna)

   Author:   Rony G. Flatscher, University of Augsburg, WU Wien University,
             (cf. "http://nestroy.wi-inf.uni-essen.de/Lv/seminare/ws0001/index-e.html");
*/
if rxFuncQuery("BSF") then  /* no bsf4rexx support available yet, hence load Java */
do
   call rxFuncAdd "BsfLoadFuncs", "BSF4Rexx", "BsfLoadFuncs"
   call BsfLoadFuncs    /* load all BSF-functions                    */
   call BsfLoadJava     /* load the JVM and set up BSF4Rexx-support  */
end

if      BsfInvokedBy()=1 then say "This Rexx program was invoked by Java!"
else if BsfInvokedBy()=2 then say "This Rexx program was invoked by Rexx, JVM loaded by Rexx!"
else                          say "No JVM present, we got troubles ..."

say "...arrived in Rexx!"
call time "r"
numeric digits 10

call setup_euro_infos   /* setup Euro-currency information      */
call setup_java_awt_gui /* setup awt-GUI        */

say "...finished with setting up, time needed:" time("e")

do forever      /* wait and fetch the text to be sent by Java           */
   a = bsf("pollEventText")     /* wait for an eventText to be sent     */
   say "debug: eventText from Java: ["a"]"
   interpret a                  /* execute as a Rexx program    */
   if result= "SHUTDOWN, REXX !" then leave     /* JVM will be shutdown in 1/10sec      */
end

exit



     /* action handler, if any key was typed in the input test-field */
key_Typed : procedure expose euro.
  parse arg input
  call dothework input  /* now do the work ...  */
  return


    /* make choice and return appropriate argument      */
choose:procedure
  if arg(1)=1 then return arg(2)
              else return arg(3)


        /* set a stem-array with the rates of exchanges */
setup_euro_infos : PROCEDURE EXPOSE euro.     /* expose stem, i.e. belongs to caller  */
    euro.ats =    13.760300;  euro.ATS.eCountry = "ATS: Austria (Schilling)"
    euro.bef =    40.339900;  euro.BEF.eCountry = "BEF: Belgium (Franc)"
    euro.dem =     1.955830;  euro.DEM.eCountry = "DEM: Germany (Mark)"
    euro.eur =     1.000000;  euro.EUR.eCountry = "EUR: Europe (Euro)"
    euro.fim =     5.945730;  euro.FIM.eCountry = "FIM: Finnland (Mark)"
    euro.frf =     6.559570;  euro.FRF.eCountry = "FRF: France (Franc)"
    euro.iep =     0.787564;  euro.IEP.eCountry = "IEP: Ireland (Pound)"
    euro.itl =  1936.270000;  euro.ITL.eCountry = "ITL: Italy (Lire)"
    euro.luf =    40.339900;  euro.LUF.eCountry = "LUF: Luxembourg (Franc)"
    euro.nlg =     2.203710;  euro.NLG.eCountry = "NLG: Netherland (Gulden)"
    euro.pte =   200.482000;  euro.PTE.eCountry = "PTE: Portugal (Escudo)"
    euro.esp =   166.386000;  euro.ESP.eCountry = "ESP: Spain (Peseta)"
    euro.grd =   340.750000;  euro.GRD.eCountry = "GRD: Greece (Drachmen)"

        /* determine processing order of currencies, will also be used as index
           into "euro."-stem, hence UPPERCASE (!) spelling     */
    euro.eOrder = "EUR ATS BEF DEM ESP FIM FRF GRD IEP ITL LUF NLG PTE"
    RETURN


        /* whenever a keytype occurred recalc values    */
DoTheWork : procedure expose euro.
  parse arg input

  /* parse first three words, translate them into uppercase       */
        /* bug in OS/2's JRE 1.1.8: value of a TextField only set after first
           call which retrieves the value of the TextField, hence a dummy call first */
call bsf 'invoke', input, 'getText'

  PARSE UPPER VALUE bsf('invoke', input, 'getText') WITH Amount InputCurrency .

        /* RGF - clear message fields */
  call bsf 'invoke', 401,  "setText", "str", date("S") time()
  call bsf 'invoke', 402,  "setText", "str", ""

  IF Amount = "" | Amount = 1 THEN      /* nothing given, show exchange rates  */
  DO
     call display 1     /* display value for 1 Euro     */
     call bsf 'invoke', 402,  "setText", "str","(Exchange rates)"
     return
  END

  IF DATATYPE(Amount, "Number") = 0 THEN
  DO
     call bsf 'invoke', 402,  "setText", "str","Input Currency not a number!"
     call beep 500, 10
     RETURN
  END

  IF InputCurrency = "" THEN InputCurrency = "EUR"      /* default      */
  ELSE IF wordpos(InputCurrency, euro.eOrder) = 0 THEN
  DO
     call bsf 'invoke', 402, "setText", "str", "Unknown currency given!"
     call beep 200, 10
     RETURN
  END
  call display amount/euro.InputCurrency        /* display values       */
  return


  /* display currency values for given euro-amount      */
display : procedure expose euro.
  parse arg euro_betrag


  betrag_id=201
  do i=201 to 201+words(euro.eOrder)-1

     tmpEuro = word(euro.eOrder, i-201+1)
     IF euro_betrag = 1 THEN stellen=6  /* rate of exchange ? */
                        ELSE stellen=2

     call bsf 'invoke', i, "setText", "str", FORMAT(euro_betrag*euro.tmpEuro,,stellen)
  END
  return


    /* create and setup up awt-objects (a frame window, a button, labels and
       an input text field; add event listeners for interesting events  */
setup_java_awt_gui : procedure expose euro.
        /* import some Java class       */
  .bsf~import('javaLabel', 'java.awt.Label')

        /* create a color               */
  bgColour = .bsf~new('java.awt.Color', 'I', '150', 'I', '150', 'I', '250')

        /* create and size frame window */
  frame  = .BSF~new( 'java.awt.Frame', 'String', 'EuroCalculator')
  Call BSF 'invoke', frame, 'setBackground', 'Object', bgColour

        /* remove layout manager or *strange* things happen ... */
  frame ~~setLayout('obj', "")
  frame ~~show ~~hide           -- show and hide frame, now insets are known
  insets=  frame~getInsets      -- get and parse insets
  parse value insets~toString with . "top=" top "," "left=" left "," "bottom=" bottom "," "right=" right "]" .

  w_width =660                  -- desired inner width
  w_col1  = 10                  -- start of first column
  w_col2  = w_width/2+w_col1    -- start of second column
  w_height=300                  -- desired inner height

  tot_width =w_width +left+right        -- total_width (insets included)
  tot_height=w_height+top +bottom       -- total_height (insets included)

  frame ~~setSize('I', tot_width, 'I',tot_height) -
        ~~setResizable("bool", .false)

        /* get physical screen dimensions and center frame */
  dim        = frame~getToolkit~getScreenSize
  scr_height = dim~bsf.getFieldValue('height')
  scr_width  = dim~bsf.getFieldValue('width')

  frame~setLocation( 'i', (scr_width-tot_width)%2 ,'i', 0 )

        /* create and set labels, then add them to the frame    */
        /* get constant frame values                            */
  label_left  = .bsf~getStaticValue( 'java.awt.Label', 'LEFT'  )
  label_right = .bsf~getStaticValue( 'java.awt.Label', 'RIGHT' )
  label_center= .bsf~getStaticValue( 'java.awt.Label', 'CENTER')

  do id=201 to 213      /* id of amount-label, appropriate currency-label       */
     if id=201 | id=208 then y=10 /* reset y-coordinate   */
     do j=0 to 100 by 100
        name=id+j       -- define name of this label
                -- create Java object under specified name (a number in this case)
        call BSF  'registerBean', name, 'java.awt.Label'
                -- wrap the Java object into an Object Rexx BSF_PROXY
        label=.BSF_Proxy~new(name)
                -- now use the Java object as if it was a Rexx object
        label ~setAlignment('Int', choose(j=0, label_right, label_left))
        label ~setSize('Int', choose(j=0, 115, 175), 'Int', 16)

                -- first eight currencies on left column, rest in right column
        x=choose(id<208, w_col1, w_col2) + choose(j=0, 0, 120)  /* add offset if necessary */
        label ~setLocation('int', x+left, 'int', y+top)

                -- set values (exchange rates)
        idx=word(euro.eOrder, right(id,2)) -- get appropriate index
        label~setText('str', choose(j=0, euro.idx, euro.idx.eCountry))

                -- add label to window
        frame~add('Obj', label)
     end

     y=y+20
  end

        /* create and set the two message labels        */
  do id=401 to 402
        -- create Java object procedurally, wrap it up in an Object Rexx proxy,
        -- can therefore be used in procedure calls as well as with messages
     label=.bsf_proxy~new(bsf('registerBean', id, 'java.awt.Label'))
     label~setAlignment( 'int', label_center )
     call bsf 'invoke', label, 'setSize',      'int', w_width-w_col1-left-right, 'int', 16
     call bsf 'invoke', label, 'setLocation',  'int', w_col1+left, 'int', 170+choose(id=401,0,20)+top
     frame~add('obj', label)
  end



        /* create and set the input field       */
  input=.bsf~new('java.awt.TextField')
  inp_width=200
  input~~setSize('int', inp_width, 'int', 20) -
       ~~setLocation('int', (w_width-inp_width)/2+left, 'int', 220+top)
  frame~add('obj', input)

        /* create the leave push-button         */
  leave =.bsf~new('java.awt.Button', 'String', 'Leave' )
  leave~~setSize('int', 60, 'int', 20) -
       ~~setLocation('int', w_col2-40+left, 'int', 260+top)
  frame~add('obj', leave)

  frame~show            -- show frame
  input~requestFocus    -- set focus to input-field

        /* define the events we are interested in and the text to be
             sent to us by Java , retrievable via "BSFPollEvents()"       */
  call BSF 'addEventListener', input,  'key',    'keyTyped',    'call key_typed "' || input || '"'
        /* get all action events        */
  call BSF 'addEventListener', leave,  'action', '',            'call BSF "exit"'
  call BSF 'addEventListener', frame, 'window', 'windowClosing','call BSF "exit"'

  return

::requires "BSF.cls"    -- import Object Rexx (syntax) support
